
package prefixnotationparser;

import java.io.IOException;
import java.io.InputStream;

/**
 *
 * @author Chris
 */
public class PrefixNotationParser
{

    private int lookaheadToken;    
    private InputStream in;  
    private String prefixExp;


    public PrefixNotationParser(InputStream in) throws IOException
    {
        this.in = in;
        lookaheadToken = in.read();
        prefixExp = "";
    }
    

    private void consume(int symbol) throws IOException, ParseError
    {
        if (lookaheadToken != symbol)
        {
            throw new ParseError();
        }
        
        lookaheadToken = in.read();
    }

    private String Factor() throws IOException, ParseError
    {      
        if ((lookaheadToken < '0' || lookaheadToken > '9') && lookaheadToken != '(')
        {
            throw new ParseError();
        }              
        else if(lookaheadToken == '(')
        {
            consume('(');
            
            return Exp();            
        }               
        else
        {
            // do nothing
            return (char) lookaheadToken + "";
        }
    }    
    
    private String Term2(String leftOperand) throws IOException, ParseError
    {
        if(lookaheadToken == '+' || lookaheadToken == '-' || lookaheadToken == ')' ||
           lookaheadToken == '\n' || lookaheadToken == -1)
        {
            // empty string - e
            return leftOperand;
        }
        else if(lookaheadToken == '*' || lookaheadToken == '/')
        {                          
            String op = (char) lookaheadToken + "";
            consume(lookaheadToken);                        
                        
            String factor = Factor();  
            
            consume(lookaheadToken);
            
            return Term2("(" + op + " " + leftOperand + " " + factor + ") ");
        }
        else
        {
            throw new ParseError();
        }
    }    

    private String Term() throws IOException, ParseError
    {
        if ((lookaheadToken < '0' || lookaheadToken > '9') && lookaheadToken != '(')
        {
            throw new ParseError();
        }        
        
        String factor = Factor();  
        
        consume(lookaheadToken);
        
        String term2 = Term2(factor);       
        
        return term2;
    }    
    
    private String Exp2(String leftOperand) throws IOException, ParseError
    {
        if(lookaheadToken == ')' ||
           lookaheadToken == '\n' || lookaheadToken == -1)
        {
            // empty string - e        
            return leftOperand;
        }
        else if(lookaheadToken == '+' || lookaheadToken == '-')
        {                 
            String op = (char) lookaheadToken + "";
            consume(lookaheadToken);                       
                        
            String term = Term();                 
            
            return Exp2("(" + op + " " + leftOperand + " " + term + ") ");
        }
        else
        {
            throw new ParseError();
        }        
    }      
    
    private String Exp() throws IOException, ParseError
    {
        if ((lookaheadToken < '0' || lookaheadToken > '9') && lookaheadToken != '(')
        {
            throw new ParseError();
        }
        
        String term = Term();           
        
        String exp2 = Exp2(term);
        
        return exp2;
    }    
    
    public void parse() throws IOException, ParseError
    {
        prefixExp = Exp();
        
        if(lookaheadToken != '\n' && lookaheadToken != -1)
        {
            throw new ParseError();
        }
    }

    
    public static void main(String[] args)
    {
        try
        {
            PrefixNotationParser parser = new PrefixNotationParser(System.in);
            
            parser.parse();
            
            System.out.println(parser.prefixExp);
        }
        catch (IOException | ParseError e)
        {
            System.err.println(e.getMessage());
        }                
    }
}
